@@ -18,7 +18,7 @@
  */
 
 #ifndef __CPUFREQD_PLUGIN_H__
-#define __CPUFREQD_PLUGIN_H__
+#define __CPUFREQD_PLUGIN_H__ 1
 
 #include <cpufreq.h>
 #include <sys/types.h>
@@ -26,6 +26,8 @@
 #include <signal.h>
 #include <unistd.h>
 #include "cpufreqd.h"
+#include "cpufreqd_remote.h"
+#include "config_parser.h"
 #include "cpufreqd_log.h"
 
 #define FALSE	0
@@ -42,7 +44,6 @@
  *  Shared struct containing useful global informations
  *  probed from the core cpufreqd at noostrap time.
  */
-
 struct cpufreq_limits {
 	unsigned long min;
 	unsigned long max;
@@ -57,9 +58,10 @@ struct cpufreq_sys_info {
 struct cpufreqd_info {
 	unsigned int kernel_version;
 	unsigned int cpus;
-	struct cpufreq_policy *cur_policy;
+	int cpufreqd_mode; /* operation mode (manual / dynamic) */
 	struct cpufreq_limits *limits;
 	struct cpufreq_sys_info *sys_info;
+	struct profile **current_profiles;
 	/* last update, IOW las call to cpufreqd_loop (see main.h)*/
 	struct timeval timestamp;
 };
@@ -67,6 +69,8 @@ struct cpufreqd_info * get_cpufreqd_info (void);
 
 struct cpufreqd_plugin;
 
+struct rule;
+
 /* 
  *  A cpufreqd keyword consists of the proper word to match at the
  *  beginning of the Rule line. The struct consists of two other function
@@ -113,7 +117,7 @@ struct cpufreqd_keyword {
 	 * Can be NULL.
 	 */
 	void (*profile_pre_change) (void *obj, const struct cpufreq_policy *old,
-			const struct cpufreq_policy *new);
+			const struct cpufreq_policy *new, const unsigned int cpu);
 
 	/* function pointer to the profile_post_change event. The same as
 	 * profile_pre_change applies except for the fact that everything is
@@ -122,7 +126,7 @@ struct cpufreqd_keyword {
 	 * Can be NULL.
 	 */
 	void (*profile_post_change) (void *obj, const struct cpufreq_policy *old,
-			const struct cpufreq_policy *new);
+			const struct cpufreq_policy *new, const unsigned int cpu);
 
 	/* function pointer to the rule_pre_change event. obj is the structure
 	 * previously provided by the parse function, old and new are the old
@@ -135,8 +139,8 @@ struct cpufreqd_keyword {
 	 *
 	 * Can be NULL.
 	 */
-	void (*rule_pre_change) (void *obj, const struct cpufreq_policy *old,
-			const struct cpufreq_policy *new);
+	void (*rule_pre_change) (void *obj, const struct rule *old,
+			const struct rule *new);
 
 	/* function pointer to the rule_post_change event. The same as
 	 * rule_pre_change applies except for the fact that everything is
@@ -144,8 +148,8 @@ struct cpufreqd_keyword {
 	 *
 	 * Can be NULL.
 	 */
-	void (*rule_post_change) (void *obj, const struct cpufreq_policy *old,
-			const struct cpufreq_policy *new);
+	void (*rule_post_change) (void *obj, const struct rule *old,
+			const struct rule *new);
 
 
 	/* Allows the owner to define a specific function to be called when freeing
