@@ -44,7 +44,7 @@
 #include "plugin_utils.h"
 #include "sock_utils.h"
 
-#define TRIGGER_EVENT(event_func, directives, dir, old, new) \
+#define TRIGGER_RULE_EVENT(event_func, directives, dir, old, new) \
 do { \
 	LIST_FOREACH_NODE(__node, (directives)) { \
 		dir = (struct directive *)__node->content; \
@@ -55,13 +55,22 @@ do { \
 	} \
 } while (0);
 
+#define TRIGGER_PROFILE_EVENT(event_func, directives, dir, old, new, cpu_num) \
+do { \
+	LIST_FOREACH_NODE(__node, (directives)) { \
+		dir = (struct directive *)__node->content; \
+		if (dir->keyword->event_func != NULL) { \
+			clog(LOG_DEBUG, "Triggering " #event_func " for %s\n", dir->keyword->word); \
+			dir->keyword->event_func(dir->obj, (old), (new), (cpu_num)); \
+		} \
+	} \
+} while (0);
+
 static struct cpufreqd_info *cpufreqd_info;
 static struct rule *current_rule;
-static struct profile *current_profile;
 static int force_reinit = 0;
 static int force_exit = 0;
 static int timer_expired = 1; /* expired in order to run on the first loop */
-static int cpufreqd_mode = MODE_DYNAMIC; /* operation mode (manual / dynamic) */
 
 /* default configuration */
 static struct cpufreqd_conf default_configuration = {
@@ -173,55 +182,79 @@ static struct rule *update_rule_scores(struct LIST *rule_list) {
  * Returns always 0 (success) except if double checking is enabled and setting
  * the policy fails in which case -1 is returned.
  */
-static int cpufreqd_set_profile (struct profile *old, struct profile *new) {
+static int cpufreqd_set_profile (struct profile **old, struct profile **new) {
 	unsigned int i;
 	struct directive *d;
+	struct profile *old_profile = NULL, *new_profile = NULL;
 
-	/* profile prechange event */
-	if (new->directives.first) {
-		TRIGGER_EVENT(profile_pre_change, &new->directives, d,
-				old!=NULL? &old->policy : NULL, &new->policy);
-	}
-	/* int cpufreq_set_policy(unsigned int cpu, struct cpufreq_policy *policy) */ 
 	for (i = 0; i < cpufreqd_info->cpus; i++) {
-		if (cpufreq_set_policy(i, &(new->policy)) == 0) {
-			clog(LOG_NOTICE, "Profile \"%s\" set for cpu%d\n", new->name, i);
+		new_profile = new[i];
+
+		if (new_profile == NULL) {
+			clog(LOG_DEBUG, "No Profile available for CPU%d doing nothing.\n", i);
+			continue;
+		}
+
+		if (old != NULL)
+			old_profile = old[i];
+
+		/* profile prechange event */
+		if (new_profile->directives.first) {
+			TRIGGER_PROFILE_EVENT(profile_pre_change, &new_profile->directives, d,
+					old_profile != NULL ? &old_profile->policy : NULL,
+					&new_profile->policy, i);
+		}
+
+		/* don't even try to set the profile if it hasn't changed */
+		if (new_profile == old_profile) {
+			clog(LOG_DEBUG, "Profile unchanged (\"%s\"-\"%s\"), for CPU%d doing nothing.\n",
+					old_profile->name, new_profile->name, i);
+		}
+		/* int cpufreq_set_policy(unsigned int cpu, struct cpufreq_policy *policy) */ 
+		else if (cpufreq_set_policy(i, &(new_profile->policy)) == 0) {
+			clog(LOG_NOTICE, "Profile \"%s\" set for CPU%d\n", new_profile->name, i);
+			cpufreqd_info->current_profiles[i] = new_profile;
+
 			/* double check if everything is OK (configurable) */
 			if (configuration->double_check) {
 				struct cpufreq_policy *check = NULL;
 				check = cpufreq_get_policy(i);
-				if (check->max != new->policy.max || check->min != new->policy.min ||
-						strcmp(check->governor, new->policy.governor) != 0) {
+				if (check->max != new_profile->policy.max
+						|| check->min != new_profile->policy.min 
+						|| strcmp(check->governor, new_profile->policy.governor) != 0) {
 					/* written policy and subsequent read disagree */
 					clog(LOG_ERR, "I haven't been able to set the chosen policy "
 							"for CPU%d.\n"
 							"I set %d-%d-%s\n"
 							"System says %d-%d-%s\n",
-							i, new->policy.max, new->policy.min,
-							new->policy.governor, check->max, 
+							i, new_profile->policy.max, new_profile->policy.min,
+							new_profile->policy.governor, check->max, 
 							check->min, check->governor);
 					cpufreq_put_policy(check);
 					return -1;
 				} else {
 					clog(LOG_INFO, "Policy correctly set %d-%d-%s\n",
-							new->policy.max, new->policy.min, new->policy.governor);
+							new_profile->policy.max,
+							new_profile->policy.min,
+							new_profile->policy.governor);
 				}
 				cpufreq_put_policy(check);
-						
-			}
+			} /* end if double_check */
 		}
 		else {
-			clog(LOG_WARNING, "Couldn't set profile \"%s\" set for cpu%d\n", new->name, i);
+			clog(LOG_WARNING, "Couldn't set profile \"%s\" set for cpu%d (%d-%d-%s)\n",
+					new_profile->name, i, new_profile->policy.max, 
+					new_profile->policy.min, new_profile->policy.governor);
 			return -1;
 		}
+
+		/* profile postchange event */
+		if (new_profile->directives.first) {
+			TRIGGER_PROFILE_EVENT(profile_post_change, &new_profile->directives, d,
+					old != NULL ? &old_profile->policy : NULL,
+					&new_profile->policy, i);
+		}
 	}
-	/* profile postchange event */
-	if (new->directives.first) {
-		TRIGGER_EVENT(profile_post_change, &new->directives, d,
-				old!=NULL? &old->policy : NULL, &new->policy);
-	}
-	current_profile = new;
-	cpufreqd_info->cur_policy = &new->policy;
 	return 0;
 }
 
@@ -250,7 +283,7 @@ static int set_cpufreqd_runmode(int mode) {
 		clog(LOG_WARNING, "Unknown mode %d\n", mode);
 		return EINVAL;
 	}
-	cpufreqd_mode = mode;
+	cpufreqd_info->cpufreqd_mode = mode;
 	return 0;
 }
 
@@ -291,7 +324,7 @@ static int read_args (int argc, char *argv[]) {
 			configuration->no_daemon = 1;
 			break;
 		case 'm':
-			cpufreqd_mode = MODE_MANUAL;
+			cpufreqd_info->cpufreqd_mode = MODE_MANUAL;
 			return 0;
 		case 'V':
 			configuration->log_level = atoi(optarg);
@@ -361,9 +394,11 @@ static void pipe_handler(int signo) {
   clog(LOG_NOTICE, "Caught PIPE signal (%s).\n", strsignal(signo));
 }
 
-static struct rule *cpufreqd_loop(struct cpufreqd_conf *conf, struct rule *current) {
+static void cpufreqd_loop(struct cpufreqd_conf *conf) {
+	int rule_equivalent = 0, ret = 0;
+	unsigned int i = 0;
 	struct rule *best_rule = NULL;
-	struct directive *d;
+	struct directive *d = NULL;
 	
 	/* update timestamp */
 	if (gettimeofday(&cpufreqd_info->timestamp, NULL) < 0) {
@@ -381,51 +416,70 @@ static struct rule *cpufreqd_loop(struct cpufreqd_conf *conf, struct rule *curre
 	if (best_rule == NULL) {
 		clog(LOG_WARNING, "No Rule matches current system status.\n");
 
-	} else if (current != best_rule) {
+	} else if (current_rule != best_rule) {
+		
 		/* rule changed */
 
 		/* try to be conservative, if the new rule has the same score
-		 * as the old one then keep the old one
+		 * as the old one then keep the old one.
+		 * Check for profiles equivalence too.
 		 */
-		if (current != NULL && current->score == best_rule->score) {
-			clog(LOG_INFO, "New Rule (\"%s\") is equivalent "
-					"to the old one (\"%s\"), doing nothing.\n",
-					best_rule->name, current->name);
-			return current;
+		if (current_rule != NULL && current_rule->score == best_rule->score) {
+
+			rule_equivalent = 1;
+			for (i = 0; i < cpufreqd_info->cpus; i++) {
+				/*
+				 * if the new rule sets a profile for a cpu not 
+				 * covered by the current rule then prefer the new rule
+				 */
+				 if (current_rule->prof[i] == NULL && best_rule->prof[i] != NULL) {
+					 rule_equivalent = 0;
+					 break;
+				 }
+			}
+			if (rule_equivalent) {
+				clog(LOG_INFO, "New Rule (\"%s\") is equivalent "
+						"to the old one (\"%s\"), doing nothing.\n",
+						best_rule->name, current_rule->name);
+				return;
+			}
 		}
 		
 		clog(LOG_DEBUG, "New Rule (\"%s\"), applying.\n", 
 				best_rule->name);
 		/* pre change event */
 		if (best_rule->directives.first != NULL) {
-			TRIGGER_EVENT(rule_pre_change, &best_rule->directives, d,
-					current != NULL ? &current->prof->policy : NULL,
-					&best_rule->prof->policy);
+			TRIGGER_RULE_EVENT(rule_pre_change, &best_rule->directives, d,
+					current_rule, best_rule);
 		}
 
 		/* change frequency */
-		if (current == NULL) {
-			cpufreqd_set_profile(NULL, best_rule->prof);
-		} else if (best_rule->prof != current->prof) {
-			cpufreqd_set_profile(current->prof, best_rule->prof);
-		} else {
-			clog(LOG_DEBUG, "Profile unchanged (\"%s\"-\"%s\"), doing nothing.\n", 
-					current->prof->name, best_rule->prof->name);
+		if (current_rule == NULL)
+			ret = cpufreqd_set_profile(NULL, best_rule->prof);
+
+		else if (best_rule->prof != current_rule->prof)
+			ret = cpufreqd_set_profile(current_rule->prof, best_rule->prof);
+
+		if (ret < 0) {
+			clog(LOG_ERR, "Cannot set policy, Rule unchanged (\"%s\").\n", 
+					current_rule != NULL ? current_rule->name : "none");
+			return;
 		}
 
 		/* post change event */
 		if (best_rule->directives.first != NULL) {
-			TRIGGER_EVENT(rule_post_change, &best_rule->directives, d,
-					current != NULL ? &current->prof->policy : NULL,
-					&best_rule->prof->policy);
+			TRIGGER_RULE_EVENT(rule_post_change, &best_rule->directives, d,
+					current_rule, best_rule);
 		}
 
+		/* update current rule */
+		current_rule = best_rule;
+
 	} else {
 		/* nothing new happened */
 		clog(LOG_DEBUG, "Rule unchanged (\"%s\"), doing nothing.\n", 
-				current->name);
+				current_rule->name);
 	}
-	return best_rule;
 }
 
 /*
@@ -434,9 +488,10 @@ static struct rule *cpufreqd_loop(struct cpufreqd_conf *conf, struct rule *curre
 static void execute_command(int sock, struct cpufreqd_conf *conf) {
 	struct pollfd fds;
 	char buf[MAX_STRING_LEN];
-	unsigned int buflen = 0, counter = 0;
-	struct profile *p;
+	unsigned int buflen = 0, counter = 0, i = 0, cpus = 0;
+	struct profile *p = NULL, **pp = NULL;
 	uint32_t command = INVALID_CMD;
+
 	/* we have a valid sock, wait for command
 	 * don't wait more tha 0.5 sec
 	 */
@@ -465,14 +520,54 @@ static void execute_command(int sock, struct cpufreqd_conf *conf) {
 				clog(LOG_DEBUG, "CMD_LIST_PROFILES\n");
 				LIST_FOREACH_NODE(node, &conf->profiles) {
 					p = (struct profile *) node->content;
+					/* FIXME: the current profile is not checked
+					 * as it may well be different from each cpu.
+					 * See command CMD_CUR_PROFILES.
+					 */
+					cpus = 0;
+					for (i = 0; i < cpufreqd_info->cpus; i++) {
+						if (cpufreqd_info->current_profiles[i] == p)
+							cpus |= (1 << i);
+					}
+					/* format is:
+					 * 1 cpu applied bitmask 
+					 * 2 profile name
+					 * 3 min freq
+					 * 4 max freq
+					 * 5 active governor
+					 */
 					buflen = snprintf(buf, MAX_STRING_LEN, "%d/%s/%lu/%lu/%s\n",
-							p == current_profile ? 1 : 0,
+							cpus,
 							p->name,
 							p->policy.min, p->policy.max,
 							p->policy.governor);
 					write(sock, buf, buflen);
 				}
 				break;
+			case CMD_CUR_PROFILES:
+				clog(LOG_DEBUG, "CMD_CUR_PROFILES\n");
+				if (!cpufreqd_info->current_profiles)
+					break;
+
+				for (i = 0; i < cpufreqd_info->cpus; i++) {
+					if (!cpufreqd_info->current_profiles[i])
+						continue;
+					/* format is:
+					 * 1 cpu where the profile is active
+					 * 2 profile name
+					 * 3 min freq
+					 * 4 max freq
+					 * 5 active governor
+					 */
+					buflen = snprintf(buf, MAX_STRING_LEN, "%d/%s/%lu/%lu/%s\n",
+							i,
+							cpufreqd_info->current_profiles[i]->name,
+							cpufreqd_info->current_profiles[i]->policy.min,
+							cpufreqd_info->current_profiles[i]->policy.max,
+							cpufreqd_info->current_profiles[i]->policy.governor);
+					write(sock, buf, buflen);
+				}
+				break;
 			case CMD_SET_RULE:
 				clog(LOG_DEBUG, "CMD_SET_RULE\n");
 				clog(LOG_ALERT, "Ignoring unimplemented command %0.8x\n", command);
@@ -480,20 +575,10 @@ static void execute_command(int sock, struct cpufreqd_conf *conf) {
 			case CMD_SET_MODE:
 				clog(LOG_DEBUG, "CMD_SET_MODE\n");
 				set_cpufreqd_runmode((int)REMOTE_ARG(command));
-#if 0
-				cpufreqd_mode = REMOTE_ARG(command);
-				if (cpufreqd_mode == MODE_MANUAL) {
-					/* reset alarm */
-					if (setitimer(ITIMER_REAL, NULL, 0) < 0)
-						clog(LOG_CRIT, "Couldn't set timer: %s\n", strerror(errno));
-				} else {
-					timer_expired = 1;
-				}
-#endif
 				break;
 			case CMD_SET_PROFILE:
 				clog(LOG_DEBUG, "CMD_SET_PROFILE\n");
-				if (cpufreqd_mode == MODE_DYNAMIC) {
+				if (cpufreqd_info->cpufreqd_mode == MODE_DYNAMIC) {
 					clog(LOG_ERR, "Couldn't set profile while running "
 							"in DYNAMIC mode.\n");
 					break;
@@ -507,10 +592,26 @@ static void execute_command(int sock, struct cpufreqd_conf *conf) {
 					p = (struct profile *) node->content;
 					counter++;
 					if (counter == REMOTE_ARG(command)) {
-						cpufreqd_set_profile(NULL, p);
+
+						/* set this profile for every cpu */
+						pp = calloc(cpufreqd_info->cpus, sizeof(struct profile *));
+						if (pp == NULL) {
+							clog(LOG_ERR, "Couldn't allocate enough memory "
+									" to set profile \"%s\"\n",
+									p->name);
+							/* return immediately */
+							return;
+						}
+
+						for(i = 0; i < cpufreqd_info->cpus; i++)
+							pp[i] = p;
+
+						cpufreqd_set_profile(NULL, pp);
+						free(pp);
+
 						/* reset the current rule to let
 						 * the cpufreqd_loop set the correct
-						 * on if going back to dynamic mode
+						 * one when going back to dynamic mode
 						 */
 						current_rule = NULL;
 						counter = 0;
@@ -544,9 +645,6 @@ int main (int argc, char *argv[]) {
 	struct sigaction signal_action;
 	sigset_t old_sigmask;
 	fd_set rfds;
-#if 0
-	struct itimerval new_timer;
-#endif
 	unsigned int i = 0;
 	int cpufreqd_sock = -1, peer_sock = -1; /* input pipe */
 	char dirname[MAX_PATH_LEN];
@@ -565,6 +663,7 @@ int main (int argc, char *argv[]) {
 		goto out;
 	}
 	memset(cpufreqd_info, 0, sizeof(struct cpufreqd_info));
+	cpufreqd_info->cpufreqd_mode = MODE_DYNAMIC;
 
 	/* 
 	 *  check perms
@@ -640,13 +739,23 @@ int main (int argc, char *argv[]) {
 		(cpufreqd_info->sys_info+i)->governors = cpufreq_get_available_governors(i);
 		(cpufreqd_info->sys_info+i)->frequencies = cpufreq_get_available_frequencies(i);
 	}
+	/* 
+	 * per-cpu profiles
+	 */
+	cpufreqd_info->current_profiles = calloc(1, cpufreqd_info->cpus * sizeof(struct profile *));
+	if (cpufreqd_info->current_profiles == NULL) {
+		clog(LOG_CRIT, "Unable to allocate memory (%s), exiting.\n", strerror(errno));
+		ret = ENOMEM;
+		goto out;
+	}
+
 
 	/* SMP: with different speed cpus */
 	cpufreqd_info->limits = calloc(1, cpufreqd_info->cpus * sizeof(struct cpufreq_limits));
 	if (cpufreqd_info->limits == NULL) {
 		clog(LOG_CRIT, "Unable to allocate memory (%s), exiting.\n", strerror(errno));
 		ret = ENOMEM;
-		goto out_sys_info;
+		goto out;
 	}
 	for (i = 0; i < cpufreqd_info->cpus; i++) {
 		/* if one of the probes fails remove all the others also */
@@ -671,12 +780,12 @@ int main (int argc, char *argv[]) {
 	if (configuration->no_daemon==0 && daemonize()!=0) {
 		clog(LOG_CRIT, "Unable to go background, exiting.\n");
 		ret = ECHILD;
-		goto out_limits;
+		goto out;
 	}
 
 cpufreqd_start:
 
-	if (init_configuration(configuration, cpufreqd_info) < 0) {
+	if (init_configuration(configuration) < 0) {
 		clog(LOG_CRIT, "Unable to parse config file: %s\n", configuration->config_file);
 		ret = EINVAL;
 		goto out_config_read;
@@ -693,11 +802,10 @@ cpufreqd_start:
 			clog(LOG_ERR, "Couldn't open socket, remote controls disabled\n");
 		} else {
 			clog(LOG_INFO, "Remote controls enabled\n");
-			if (cpufreqd_mode == MODE_MANUAL)
+			if (cpufreqd_info->cpufreqd_mode == MODE_MANUAL)
 				clog(LOG_INFO, "Starting in manual mode\n");
 		}
-	} else
-		cpufreqd_mode = MODE_DYNAMIC;
+	}
 
 	/* Validate plugins, if none left exit.... */
 	if (validate_plugins(&configuration->plugins) == 0) {
@@ -732,7 +840,7 @@ cpufreqd_start:
 		sigprocmask(SIG_BLOCK, &signal_action.sa_mask, &old_sigmask);
 	}
 	
-	set_cpufreqd_runmode(cpufreqd_mode);
+	set_cpufreqd_runmode(cpufreqd_info->cpufreqd_mode);
 	/*
 	 *  Looooooooop
 	 */
@@ -741,22 +849,8 @@ cpufreqd_start:
 		 * Run the system scan and rule selection and set timer
 		 * if running in DYNAMIC mode AND the timer is expired
 		 */
-		if (cpufreqd_mode == MODE_DYNAMIC && timer_expired) {
-#if 0
-			new_timer.it_interval.tv_usec = configuration->poll_intv.tv_usec;
-			new_timer.it_interval.tv_sec = configuration->poll_intv.tv_sec;
-			new_timer.it_value.tv_usec = configuration->poll_intv.tv_usec;
-			new_timer.it_value.tv_sec = configuration->poll_intv.tv_sec;
-#endif
-			current_rule = cpufreqd_loop(configuration, current_rule);
-#if 0
-			/* set next alarm */
-			if (setitimer(ITIMER_REAL, &new_timer, 0) < 0) {
-				ret = errno;
-				clog(LOG_CRIT, "Couldn't set timer: %s\n", strerror(errno));
-				break;
-			}
-#endif
+		if (cpufreqd_info->cpufreqd_mode == MODE_DYNAMIC && timer_expired) {
+			cpufreqd_loop(configuration);
 			/* can safely reset the expired flag now */
 			timer_expired = 0;
 		}
@@ -767,7 +861,7 @@ cpufreqd_start:
 			FD_ZERO(&rfds);
 			FD_SET(cpufreqd_sock, &rfds);
 			
-			if (!timer_expired || cpufreqd_mode == MODE_MANUAL) {
+			if (!timer_expired || cpufreqd_info->cpufreqd_mode == MODE_MANUAL) {
 				switch (pselect(cpufreqd_sock+1, &rfds, NULL, NULL, NULL, &old_sigmask)) {
 					case 0:
 						/* timed out. check to see if things have changed */
@@ -826,26 +920,28 @@ out_config_read:
 		goto cpufreqd_start;
 	}
 
-out_limits:
-	if (cpufreqd_info->limits != NULL)
-		free(cpufreqd_info->limits);
-
-out_sys_info:
-	if (cpufreqd_info->sys_info != NULL) {
-		for (i=0; i<cpufreqd_info->cpus; i++) {
-			if ((cpufreqd_info->sys_info+i)->governors!=NULL)
-				cpufreq_put_available_governors((cpufreqd_info->sys_info+i)->governors);
-			if ((cpufreqd_info->sys_info+i)->affected_cpus!=NULL)
-				cpufreq_put_affected_cpus((cpufreqd_info->sys_info+i)->affected_cpus);
-			if ((cpufreqd_info->sys_info+i)->frequencies!=NULL)
-				cpufreq_put_available_frequencies((cpufreqd_info->sys_info+i)->frequencies);
-		}
-		free(cpufreqd_info->sys_info);
-	}
-
 out:
-	if (cpufreqd_info != NULL)
+	if (cpufreqd_info != NULL) {
+		if (cpufreqd_info->limits != NULL)
+			free(cpufreqd_info->limits);
+
+		if (cpufreqd_info->sys_info != NULL) {
+			for (i=0; i<cpufreqd_info->cpus; i++) {
+				if ((cpufreqd_info->sys_info+i)->governors!=NULL)
+					cpufreq_put_available_governors((cpufreqd_info->sys_info+i)->governors);
+				if ((cpufreqd_info->sys_info+i)->affected_cpus!=NULL)
+					cpufreq_put_affected_cpus((cpufreqd_info->sys_info+i)->affected_cpus);
+				if ((cpufreqd_info->sys_info+i)->frequencies!=NULL)
+					cpufreq_put_available_frequencies((cpufreqd_info->sys_info+i)->frequencies);
+			}
+			free(cpufreqd_info->sys_info);
+		}
+
+		if (cpufreqd_info->current_profiles != NULL)
+			free(cpufreqd_info->current_profiles);
+
 		free(cpufreqd_info);
+	}
 	if (configuration != NULL)
 		free(configuration);
 	/*
